import {
  Body,
  Controller,
  Get,
  HttpCode,
  HttpStatus,
  Post,
  UploadedFiles,
  UseGuards,
  UseInterceptors
} from "@nestjs/common";
import { AdminAuthGuard } from "src/core/guards/admin-auth.guard";
import { Constants } from "src/utils/constants";
import { ContactUsPageCmsService } from "./contact-us-page-cms.service";
import { UpdateContactUsPageCmsDto } from "./dto/update-contact-us-page-cms.dto";
import { FileFieldsInterceptor } from "@nestjs/platform-express";
import { UploadedMulterFileI } from "../../SpacesModule/SpacesService";

@Controller({ path: "contact-us-page-cms", version: Constants.API_VERSION_1 })

export class ContactUsPageCmsController {
  constructor(private readonly service: ContactUsPageCmsService) {
  }

  @Post()
  @HttpCode(HttpStatus.OK)
  @UseGuards(AdminAuthGuard)
  @UseInterceptors(FileFieldsInterceptor(
    [
      { name: "image", maxCount: 1 }
    ]
  ))
  updateOrCreateData(
    @Body() dto: UpdateContactUsPageCmsDto,
    @UploadedFiles() files: {
      image?: UploadedMulterFileI
    }
  ) {
    return this.service.updateOrCreateData(dto, files);
  }

  @Get()
  // @UseGuards(AdminAuthGuard)
  findAll() {
    return this.service.findAll();
  }
}
