import { EntityRepository, Repository } from "typeorm";
import { UpdateShopPageComponentUiDto } from "./dto/update-shop-page-component-ui.dto";
import { ShopPageComponentUiEntity } from "./entities/shop-page-component-ui.entity";

@EntityRepository(ShopPageComponentUiEntity)
export class ShopPageComponentUiRepository extends Repository<ShopPageComponentUiEntity> {

  async createEntity(dto: UpdateShopPageComponentUiDto): Promise<ShopPageComponentUiEntity | Error> {
    try {
      const entity = this.create({
        backgroundColor: dto.backgroundColor,
        pageTitleFontColor: dto.pageTitleFontColor,
        brandSectionTitleFontColor: dto.brandSectionTitleFontColor,
        categorySectionTitleFontColor: dto.categorySectionTitleFontColor,
        flowerSectionTitleFontColor: dto.flowerSectionTitleFontColor
      });
      await this.save(entity);
      return entity;
    } catch (e) {
      return e;
    }
  }

  async updateEntity(dto: UpdateShopPageComponentUiDto, id: string): Promise<ShopPageComponentUiEntity | Error> {
    try {
      await this.update(id, {
        backgroundColor: dto.backgroundColor,
        pageTitleFontColor: dto.pageTitleFontColor,
        brandSectionTitleFontColor: dto.brandSectionTitleFontColor,
        categorySectionTitleFontColor: dto.categorySectionTitleFontColor,
        flowerSectionTitleFontColor: dto.flowerSectionTitleFontColor
      });
      return await this.findOne(id);
    } catch (e) {
      return e;
    }
  }

  async findAll(): Promise<ShopPageComponentUiEntity[] | Error> {
    try {
      return await this.find({
        where: {
          isDeleted: false
        }
      });
    } catch (e) {
      return e;
    }
  }
}