import {
  Body,
  Controller,
  Delete,
  Get,
  Param,
  Patch,
  Post,
  UploadedFiles,
  UseGuards,
  UseInterceptors
} from "@nestjs/common";
import { Constants } from "src/utils/constants";
import { DispoDifferenceSectionService } from "./dispo-difference-section.service";
import { CreateDispoDifferenceSectionDto } from "./dto/create-dispo-difference-section.dto";
import { UpdateDispoDifferenceSectionDto } from "./dto/update-dispo-difference-section.dto";
import { AdminAuthGuard } from "src/core/guards/admin-auth.guard";
import { FileFieldsInterceptor } from "@nestjs/platform-express";
import { UploadedMulterFileI } from "../SpacesModule/SpacesService";

@Controller({ path: "dispo-difference-section", version: Constants.API_VERSION_1 })

export class DispoDifferenceSectionController {
  constructor(private readonly service: DispoDifferenceSectionService) {
  }

  @Post()
  @UseGuards(AdminAuthGuard)
  @UseInterceptors(FileFieldsInterceptor(
    [
      { name: "image", maxCount: 1 }
    ]
  ))
  create(@Body() dto: CreateDispoDifferenceSectionDto,
         @UploadedFiles() files: {
           image: UploadedMulterFileI,
         }) {
    return this.service.create(dto, files);
  }

  @Get()
  @UseGuards(AdminAuthGuard)
  findAll() {
    return this.service.findAll();
  }

  @Get(":id")
  @UseGuards(AdminAuthGuard)
  findOne(@Param("id") id: string) {
    return this.service.findOne(id);
  }

  @Patch("/:id")
  @UseGuards(AdminAuthGuard)
  @UseInterceptors(FileFieldsInterceptor(
    [
      { name: "image", maxCount: 1 }
    ]
  ))
  update(
    @Param("id") id: string,
    @Body() dto: UpdateDispoDifferenceSectionDto,
    @UploadedFiles() files: {
      image: UploadedMulterFileI,
    }) {
    return this.service.update(id, dto, files);
  }

  @Delete(":id")
  @UseGuards(AdminAuthGuard)
  async remove(@Param("id") id: string) {
    return await this.service.remove(id);
  }
}
