import { Injectable } from "@nestjs/common";
import { getGraphQLClientWithApiKey } from "src/helper/graphql-client-helper";
import {
    fetchAllProduct,
    fetchAllProductBrand,
    fetchAllProductCategory,
    fetchAllProductEffects,
    fetchAllProductStrainType,
    fetchOneProduct,
    fetchProductsCount,
    filterListQuery,
    filterProducts,
    menuType
} from "src/helper/dutchie-plus-query";
import { InjectRepository } from "@nestjs/typeorm";
import {
    ProductCategoryRepository
} from "src/modules/dutchie-plus/custom/product-category/product-category.repository";
import { RetailerServiceRepository } from "../custom/retailer-service/retailer-service.repository";
import { ProductServiceRepository } from "../custom/product-service/product-service.repository";
import { ProductEffectsRepository } from "../custom/product-effects/product-effects.repository";


@Injectable()
export class ProductService {
  constructor(
    @InjectRepository(ProductCategoryRepository)
    private readonly productCatRepo: ProductCategoryRepository,
    @InjectRepository(RetailerServiceRepository)
    private readonly retailerServiceRepo: RetailerServiceRepository,
    @InjectRepository(ProductServiceRepository)
    private readonly productServiceRepo: ProductServiceRepository,
    @InjectRepository(ProductEffectsRepository)
    private readonly productEffectsRepo: ProductEffectsRepository
  ) {
  }

  async findAll(variables: any) {
    try {
      const retailerService = await this.retailerServiceRepo.findOne({
        where: {
          retailerId: variables.retailerId,
          isDeleted: false
        }
      });
      if (retailerService) {
        const graphQLClient = getGraphQLClientWithApiKey(retailerService.apiKeyName);
        const query = fetchAllProduct();
        const data = await graphQLClient.request(query, variables);
        return data;
      } else {
        return [];
      }
    } catch (e) {
      return e;
    }
  }

  async findOne(variables: any) {
    try {
      const retailerService = await this.retailerServiceRepo.findOne({
        where: {
          retailerId: variables.retailerId,
          isDeleted: false
        }
      });
      if (retailerService) {
        const graphQLClient = getGraphQLClientWithApiKey(retailerService.apiKeyName);
        const query = fetchOneProduct();
        const data = await graphQLClient.request(query, variables);
        return data;
      } else {
        return [];
      }
    } catch (e) {
      return e;
    }
  }

  async findAllBrands(variables: any) {
    try {
      const retailerService = await this.retailerServiceRepo.findOne({
        where: {
          retailerId: variables.retailerId,
          isDeleted: false
        }
      });
      if (retailerService) {
        const graphQLClient = getGraphQLClientWithApiKey(retailerService.apiKeyName);
        const query = fetchAllProductBrand();
        const data = await graphQLClient.request(query, variables);
        return data["menu"];
      } else {
        return [];
      }
    } catch (e) {
      return e;
    }
  }

  async filterProducts(
    variables: any
  ) {
    try {
      const retailerService = await this.retailerServiceRepo.findOne({
        where: {
          retailerId: variables.retailerId,
          isDeleted: false
        }
      });
      if (retailerService) {
        const graphQLClient = getGraphQLClientWithApiKey(retailerService.apiKeyName);
        const query = filterProducts();
        const data = await graphQLClient.request(query, variables);

        // const filterListQ = filterListQuery();
        // const filterListReq = await graphQLClient.request(filterListQ, variables);
        // const filterList = filterListReq["menu"].products;

        // // find all category
        // let category = [...new Set(filterList.map(item => item.category))];
        // // working with strain Type
        // let strainType = [...new Set(filterList.map(item => item.strainType))];
        // // working with effects
        // let effects = []
        // for (let i = 0; i < filterList.length; i++) {
        //     filterList[i].effects.forEach(element => {
        //         if (!effects.includes(element)) {
        //             effects.push(element);
        //         }
        //     });
        // }

        // // retailer menu type
        // const menuquery = menuType();
        // const menuTypeReq = await graphQLClient.request(menuquery, {id: variables.retailerId});
        // let menuTypeList = [];
        // if (menuTypeReq["retailer"]) {
        //     menuTypeList = menuTypeReq["retailer"].menuTypes;
        // }

        // fetch sub category based on category
        let subCategory = [];
        if (variables.filter.category && data["menu"].products.length > 0) {
          subCategory = [...new Set(data["menu"].products.map(item => item.subcategory))];
        }

        // assign it to data
        if (data["menu"] && typeof variables.maxPrice === "undefined" && typeof variables.minPrice === "undefined") {
          // data["menu"]["category"] = category;
          // data["menu"]["strainType"] = strainType;
          // data["menu"]["effects"] = effects;
          // data["menu"]["menuType"] = menuTypeList;
          data["menu"]["subCategory"] = subCategory;
        }

        // filter price
        const productCount = data["menu"].productsCount; // PRODUCT COUNT
        if (typeof variables.maxPrice !== "undefined" && typeof variables.minPrice !== "undefined") {
          const minPrice = variables.minPrice;
          const maxPrice = variables.maxPrice;
          let variableObject = {
            "retailerId": variables.retailerId,
            "filter": variables.filter,
            "pagination": {
              "offset": 0,
              "limit": productCount
            }
          };
          if (typeof variables.menuType !== "undefined") variableObject["menuType"] = variables.menuType;

          // fetch all product without default pagination
          const query = filterProducts();
          const data = await graphQLClient.request(query, variableObject);

          const response = data["menu"].products.filter(function(x: any) {
            return x.variants[0].priceMed >= minPrice && x.variants[0].priceMed <= maxPrice;
          });

          data["menu"]["productsCount"] = response.length;
          data["menu"]["products"] = response.splice(variables.pagination.offset, variables.pagination.limit);

          // assign it to data
          if (data["menu"]) {
            // data["menu"]["category"] = category;
            // data["menu"]["strainType"] = strainType;
            // data["menu"]["effects"] = effects;
            // data["menu"]["menuType"] = menuTypeList;
            data["menu"]["subCategory"] = subCategory;
          }
          return data;
        } else {
          return data;
        }
      } else {
        return [];
      }
    } catch (e) {
      return e;
    }
  }

  async findAllCategory(variables: any) {
    try {
      const retailerService = await this.retailerServiceRepo.findOne({
        where: {
          retailerId: variables.retailerId,
          isDeleted: false
        }
      });
      if (retailerService) {
        const graphQLClient = getGraphQLClientWithApiKey(retailerService.apiKeyName);
        const query = fetchAllProductCategory();
        const data = await graphQLClient.request(query, variables);
        return data["menu"];
      } else {
        return [];
      }
    } catch (e) {
      return e;
    }
  }

  async findUniqueCategory(variables: any) {
    try {
      const retailerService = await this.retailerServiceRepo.findOne({
        where: {
          retailerId: variables.retailerId,
          isDeleted: false
        }
      });
      if (retailerService) {
        const graphQLClient = getGraphQLClientWithApiKey(retailerService.apiKeyName);
        const query = fetchAllProductCategory();
        const data = await graphQLClient.request(query, variables);
        let categoryData = data["menu"].products;
        let uniqueCategory = [...new Set(categoryData.map(item => item.category))];

        // get category image from product category table
        let productUniqCatList = [];
        for (let i = 0; i < uniqueCategory.length; i++) {
          const productCat = await this.productCatRepo.findOneCategory(uniqueCategory[i]);
          if (productCat) {
            const obj = {
              category: productCat.categoryName,
              image: productCat.image,
              featuredImage: productCat.featuredImage
            };
            productUniqCatList.push(obj);
          }
        }
        return productUniqCatList;
      } else {
        return [];
      }
    } catch (e) {
      return e;
    }
  }

  async findAllStrainType(variables: any) {
    try {
      const retailerService = await this.retailerServiceRepo.findOne({
        where: {
          retailerId: variables.retailerId,
          isDeleted: false
        }
      });
      if (retailerService) {
        const graphQLClient = getGraphQLClientWithApiKey(retailerService.apiKeyName);
        const query = fetchAllProductStrainType();
        const data = await graphQLClient.request(query, variables);
        return data["menu"];
      } else {
        return [];
      }
    } catch (e) {
      return e;
    }
  }

  async findAllEffects(variables: any) {
    try {
      const retailerService = await this.retailerServiceRepo.findOne({
        where: {
          retailerId: variables.retailerId,
          isDeleted: false
        }
      });
      if (retailerService) {
        const graphQLClient = getGraphQLClientWithApiKey(retailerService.apiKeyName);
        const query = fetchAllProductEffects();
        const data = await graphQLClient.request(query, variables);
        return data["menu"];
      } else {
        return [];
      }
    } catch (e) {
      return e;
    }
  }

  async filterList(variables: any) {
    try {
      const retailerService = await this.retailerServiceRepo.findOne({
        where: {
          retailerId: variables.retailerId,
          isDeleted: false
        }
      });
      if (retailerService) {
        const graphQLClient = getGraphQLClientWithApiKey(retailerService.apiKeyName);
        const productCountQuery = fetchProductsCount();
        const countData = await graphQLClient.request(productCountQuery, variables);
        const filterVariable = {
          retailerId: variables.retailerId,
          pagination: {
            offset: 0,
            limit: countData.menu.productsCount
          }
        };

        const filterListQ = filterListQuery();
        const filterListResponse = await graphQLClient.request(filterListQ, filterVariable);
        const filterList = filterListResponse.menu.products;

        // Find all unique categories
        const categories = [...new Set(filterList.map(item => item.category))];

        // Find all unique subcategories
        const subCategories = [...new Set(filterList.map(item => item.subcategory))];

        // Find all unique strain types
        const strainTypes = [...new Set(filterList.map(item => item.strainType))];

        // Find all unique effects
        const effects = new Set();
        for (const filter of filterList) {
          for (const effect of filter.effects) {
            effects.add(effect);
          }
        }
        const uniqueEffects = Array.from(effects);

        // Get the weights
        const weights = filterListResponse.menu.weights;

        // Get the brands
        const brands = filterListResponse.menu.brands.map(item => {
          return {
            id: item.id,
            name: item.name
          };
        });

        // Get the menu types for the retailer
        const menuTypeQuery = menuType();
        const menuTypeResponse = await graphQLClient.request(menuTypeQuery, { id: variables.retailerId });
        const menuTypeList = menuTypeResponse.retailer ? menuTypeResponse.retailer.menuTypes : [];

        return {
          categories,
          subCategories,
          strainTypes,
          effects: uniqueEffects,
          weights,
          brands,
          menuTypes: menuTypeList
        };
      } else {
        return [];
      }
    } catch (e) {
      return e;
    }
  }

  async retailerWiseBrandCategory(variables: any) {
    try {
      const category = await this.productCatRepo.find({
        order: { categoryName: "ASC" },
        where: {
          isDeleted: false
        }
      });
      const productBrand = await this.productServiceRepo.find({
        where: {
          retailerId: variables.retailerId,
          isDeleted: false,
          isShowBrand: true
        }
      });

      const uniqueBrands = productBrand
        .map(({ brandId, retailerId, retailerName, brandName, brandImage, isShowBrand }) => ({
          id: brandId,
          retailerId,
          retailerName,
          name: brandName,
          imageUrl: brandImage,
          isShowBrand
        })) // Extract specific properties
        .filter((item, index, self) => item.name !== "" && self.findIndex((t) => t.name === item.name) === index); // Filter empty brand names and duplicates based on brand

      return {
        brands: uniqueBrands,
        category: category
      };
    } catch (e) {
      return e;
    }
  }

  async customFilterList(): Promise<any | Error> {
    const category = await this.productCatRepo.find({
      order: { categoryName: "ASC" },
      where: {
        isDeleted: false
      }
    });

    const effects = await this.productEffectsRepo.find({
      order: { effects: "ASC" },
      where: {
        isDeleted: false
      }
    });

    const strainType = await this.productServiceRepo.getProductStrainType();


    return {
      category: category.map(item => item.categoryName),
      effects: effects.map(item => item.effects),
      strainType: strainType
        .filter(item => item.strainType !== null)
        .map(item => item.strainType)
    };
  }
}
