import { EntityRepository, getConnection, Repository } from "typeorm";
import { FooterImageEntity } from "./entities/footer-image.entity";
import { CreateFooterImageDto } from "./dto/create-footer-image.dto";
import { UpdateFooterImageDto } from "./dto/update-footer-image.dto";

@EntityRepository(FooterImageEntity)
export class FooterImageRepository extends Repository<FooterImageEntity> {
  async createEntity(dto: CreateFooterImageDto): Promise<FooterImageEntity | Error> {
    try {
      const {
        image
      } = dto;
      const entity = this.create({
        image
      });
      await this.save(entity);
      return entity;
    } catch (e) {
      return e;
    }
  }

  async findOneEntity(id: string): Promise<FooterImageEntity | Error> {
    try {
      return await this.findOne(id);
    } catch (e) {
      return e;
    }
  }

  async findByFilterQuery(query: any): Promise<FooterImageEntity[] | Error> {
    try {
      return await this.find({
        where: {
          ...query,
          isDeleted: false
        },
        order: {
          createdAt: "DESC"
        }
      });
    } catch (e) {
      return e;
    }
  }

  async deleteEntity(id: string): Promise<boolean | Error> {
    try {
      const updateResult = await getConnection()
        .createQueryBuilder()
        .update(FooterImageEntity)
        .set({ isDeleted: true })
        .where("id = :id", { id: id })
        .execute();
      if (updateResult.affected > 0) {
        return true;
      }
      return false;
    } catch (e) {
      return e;
    }
  }

  async countAllQuery(): Promise<any | Error> {
    try {
      return await this.count({
        where: {
          isDeleted: false
        }
      });
    } catch (e) {
      return e;
    }
  }

  async updateEntity(id: string, dto: UpdateFooterImageDto): Promise<FooterImageEntity | Error> {
    try {
      await this.update(id, {
        image: dto.image
      });
      return await this.findOne(id);
    } catch (e) {
      return e;
    }
  }
}