import { HttpException, HttpStatus, Injectable } from "@nestjs/common";
import { InjectRepository } from "@nestjs/typeorm";
import { Constants } from "src/utils/constants";
import { ResponseUtils } from "src/utils/response.utils";
import { UploadedMulterFileI } from "../SpacesModule/SpacesService";
import { DoSpacesService } from "../SpacesModule/SpacesService/doSpacesService";
import { HeroSectionSliderRepository } from "./hero-section-slider.repository";
import { CreateHeroSectionSliderDto } from "./dto/create-hero-section-slider.dto";
import { HeroSectionSliderEntity } from "./entities/hero-section-slider.entity";
import { UpdateHeroSectionSliderDto } from "./dto/update-hero-section-slider.dto";

@Injectable()
export class HeroSectionSliderService {
  constructor(
    @InjectRepository(HeroSectionSliderRepository)
    private readonly repository: HeroSectionSliderRepository,
    private readonly doSpaceService: DoSpacesService
  ) {
  }

  async create(
    dto: CreateHeroSectionSliderDto,
    files: {
      image: UploadedMulterFileI
    }
  ): Promise<HeroSectionSliderEntity | Error> {
    if (files && files.image) {
      const image: any = await this.doSpaceService.uploadFile(files.image[0], "FaqPageSectionTwo/");
      dto.image = image;
    }
    const data = await this.repository.createEntity(dto);
    if (!data) {
      throw new HttpException(
        Constants.CREATE_FAILED,
        HttpStatus.BAD_REQUEST
      );
    }
    return ResponseUtils.successResponseHandler(201, "Data created successfully", "data", data);
  }

  async findAll(): Promise<HeroSectionSliderEntity[] | Error> {
    const data = await this.repository.findByFilterQuery({});
    if (!data) {
      throw new HttpException(
        Constants.NOT_FOUND,
        HttpStatus.NOT_FOUND
      );
    }
    return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
  }

  async findOne(id: string): Promise<HeroSectionSliderEntity[] | Error> {
    const data = await this.repository.findByFilterQuery({ id: id });
    if (!data) {
      throw new HttpException(
        Constants.NOT_FOUND,
        HttpStatus.NOT_FOUND
      );
    }
    return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
  }

  async remove(id: string): Promise<boolean | Error> {
    const data = await this.repository.deleteEntity(id);
    if (!data) {
      throw new HttpException(
        Constants.NOT_FOUND,
        HttpStatus.NOT_FOUND
      );
    }
    return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully");
  }

  async update(
    id: string,
    dto: UpdateHeroSectionSliderDto,
    files: {
      image: UploadedMulterFileI
    }): Promise<HeroSectionSliderEntity | Error> {
    const res: any = await this.repository.findOneEntity(id);
    if (!res) {
      throw new HttpException(
        Constants.NOT_FOUND,
        HttpStatus.NOT_FOUND
      );
    }

    if (files && files.image) {
      const image: any = await this.doSpaceService.uploadFile(files.image[0], "FaqPageSectionTwo/");
      dto.image = image;
    }
    const data = await this.repository.updateEntity(id, dto);
    if (!data) {
      throw new HttpException(
        Constants.UPDATE_FAILED,
        HttpStatus.BAD_REQUEST
      );
    }
    return ResponseUtils.successResponseHandler(200, "Data updated successfully", "data", data);
  }
}
