import { Controller, Get, Param } from "@nestjs/common";
import { Query, UseGuards } from "@nestjs/common/decorators";
import { AdminAuthGuard } from "src/core/guards/admin-auth.guard";
import { Constants } from "src/utils/constants";
import { SiteFrontendService } from "./site-frontend.service";

@Controller({
  path: "site",
  version: Constants.API_VERSION_1
})
export class SiteFrontendController {
  constructor(private readonly service: SiteFrontendService) {
  }

  @Get("/career")
  getCareerPageData() {
    return this.service.getCareerPageData();
  }

  @Get("/career-list")
  getCareerDepartmentWiseData(
    @Query("departmentNameSlug") departmentNameSlug: any,
    @Query("page") page: number,
    @Query("limit") limit: number
  ) {
    return this.service.getCareerDepartmentWiseData(departmentNameSlug, page, limit);
  }

  @Get("/career/:id")
  getCareerDetailsData(@Param("id") id: string) {
    return this.service.getCareerDetailsData(id);
  }

  @Get("/blog")
  getBlogData(
    @Query("page") page: number,
    @Query("limit") limit: number,
    @Query("keyword") keyword?: any,
    @Query("tagName") tagName?: any,
    @Query("categoryId") categoryId?: any,
    @Query("date") date?: Date
  ) {
    return this.service.getBlogData(page, limit, keyword, tagName, categoryId, date);
  }

  @Get("/blog-details/:urlSlug")
  getBlogDetailsData(@Param("urlSlug") urlSlug: string) {
    return this.service.getBlogDetailsData(urlSlug);
  }

  @Get("/faq")
  getFaqPageData() {
    return this.service.getFaqPageData();
  }

  @Get("/contact-us")
  getContactUsPageData() {
    return this.service.getContactUsPageData();
  }

  @Get("/about-us")
  getAboutUsPageData() {
    return this.service.getAboutUsPageData();
  }

  @Get("/home")
  home() {
    return this.service.home();
  }

  @Get("/admin-dashboard")
  @UseGuards(AdminAuthGuard)
  getDashboardData() {
    return this.service.getDashboardData();
  }

  @Get("/component-ui")
  getComponentUI() {
    return this.service.getComponentUI();
  }
}
